#include <gargoyle.h>
#include <gargoyle/scribe.h>
#include <gargoyle/sleuth.h>
#include <gargoyle/twine.h>
#include <stddef.h>

uint8_t gargoyle_digest_argv(uint16_t optc, struct gargoyle_opt *optv, int *argc, char ***argv, gargoyle_flag_type flags) {
  if(!(flags & GARGOYLE_FLG_GREED)) {
    *argc -= 1;
    *argv += 1;
  }

  if(flags & GARGOYLE_FLG_BCASE) {
    flags |= GARGOYLE_FLG_ICASE;
  }

  while(1) {
    const char *arg = **argv;

    if(!arg || *arg != '-' || !*(arg + 1)) {
      return GARGOYLE_ERR_SUCCESS;
    } else if(*arg == '-' && *(arg + 1) == '-' && !*(arg + 2)) {
      *argc -= 1;
      *argv += 1;
      return GARGOYLE_ERR_SUCCESS;
    }

    struct gargoyle_opt *opt = NULL;

    if(*(arg + 1) == '-') {
      const char *brand = arg + 2;
      opt = gargoyle_find_brand(optc, optv, brand, NULL, 0, flags);

      if(!opt) {
        if(flags & GARGOYLE_FLG_STRCT) {
          return GARGOYLE_ERR_UNKNOWN_OPT;
        } else {
          *argc -= 1;
          *argv += 1;
          continue;
        }
      }

      uint8_t res = 0;

      if(opt->type & GARGOYLE_TYPE_BOOL) {
        res = gargoyle_from_bool(opt, brand, "no-", 3, flags);
      } else if(*(brand + opt->brand_sz) == '=') {
        res = gargoyle_from_rope(opt, brand + opt->brand_sz + 1, flags);
      } else {
        if(!*(*argv + 1)) {
          return GARGOYLE_ERR_VALUE_REQUIRED;
        }

        *argc -= 1;
        *argv += 1;
        res = gargoyle_from_rope(opt, **argv, flags);
      }

      if(res) {
        return res;
      }
    } else {
      const char *idx = arg + 1;

      for(; *idx; idx += 1) {
        opt = gargoyle_find_emblem(optc, optv, *idx, flags);

        if(!opt) {
          if(flags & GARGOYLE_FLG_STRCT) {
            return GARGOYLE_ERR_UNKNOWN_OPT;
          } else {
            *argc -= 1;
            *argv += 1;
            continue;
          }
        }

        uint8_t res = 0;

        if(opt->type & GARGOYLE_TYPE_BOOL) {
          res = gargoyle_from_bool(opt, "", "*", 1, flags);
        } else if(*(idx + 1)) {
          res = gargoyle_from_rope(opt, idx + 1, flags);
          break;
        } else {
          if(!*(*argv + 1)) {
            return GARGOYLE_ERR_VALUE_REQUIRED;
          }

          *argc -= 1;
          *argv += 1;
          res = gargoyle_from_rope(opt, **argv, flags);
        }
      }
    }

    *argc -= 1;
    *argv += 1;
  }

  return GARGOYLE_ERR_SUCCESS;
}

uint8_t gargoyle_digest_envh(uint16_t optc, struct gargoyle_opt *optv, const char *prefix, uint16_t prefix_sz, char ***envh, gargoyle_flag_type flags) {
  if(flags & GARGOYLE_FLG_BCASE) {
    flags |= GARGOYLE_FLG_ICASE;
  }

  for(; **envh; *envh += 1) {
    const char *var = **envh;

    if(!gargoyle_cmp(prefix, var, prefix_sz, flags) || *(var + prefix_sz) != '_') {
      continue;
    }

    const char *brand = var + prefix_sz + 1;
    struct gargoyle_opt *opt = gargoyle_find_brand(optc, optv, brand, "NO_", 3, flags);

    if(!opt) {
      if(flags & GARGOYLE_FLG_STRCT) {
        return GARGOYLE_ERR_UNKNOWN_OPT;
      } else {
        continue;
      }
    }

    uint8_t res = 0;

    if(opt->type & GARGOYLE_TYPE_BOOL) {
      res = gargoyle_from_bool(opt, brand, "NO_", 3, flags);
    } else {
      res = gargoyle_from_rope(opt, brand + opt->brand_sz + 1, flags);
    }

    if(res) {
      return res;
    }
  }

  return 0;
}
